#ifndef BARRIERESTIMATION_HH_
#define BARRIERESTIMATION_HH_


#include "ell/LandscapeTopology.hh"
#include "ell/MinimumSelector.hh"
#include "ell/SamplingAbortionCriterion.hh"

namespace ell
{

	  /*!
	   * An abstract interface of algorithms that will perform an iteration of
	   * barrier estimations between minima of a given landscape.
	   */
	class BarrierSampling
	{
	public:
		virtual ~BarrierSampling();
		
		  /*!
		   * Performs an iteration of barrier estimations until the sac decides
		   * that no further estimations are neccessary.
		   * 
		   * @param lt the landscape topology to add the barriers to and to
		   *        get the minima from between the barrier should be estimated.
		   * @param ms selects minima between which the next barrier should be
		   *        estimated.
		   * @param sac decides if another estimation iteration should be done.
		   * 
		   * @return the number of barrier sampling iterations performed.
		   */
		virtual
		size_t
		findBarriers(	LandscapeTopology* const lt
						, MinimumSelector* const ms
						, SamplingAbortionCriterion * const sac
						) = 0;
	};

} // namespace ell

#include "ell/DirectWalk.hh"

namespace ell
{

	  /*!
	   * A barrier estimation approach using direct walks between minima
	   * to estimate a low barrier between them. The
	   * minima are choosen using the given minimum selector and the sampling
	   * is aborted by the abortion criterion.
	   * 
	   * This BarrierSampling will not find new minima.
	   */
	class BS_DirectWalk : public BarrierSampling
	{
	public:
		
		  //! construction
		BS_DirectWalk(const DirectWalk* const dw);
		
		  //! destruction
		virtual ~BS_DirectWalk();
		
		  /*!
		   * Performs an iteration of barrier estimations using direct walks
		   * between minima of the landscape until the sac decides
		   * that no further estimations are neccessary.
		   * 
		   * @param lt the landscape topology to add the barriers to and to
		   *        get the minima from between the barrier should be estimated.
		   * @param ms selects minima between which the next barrier should be
		   *        estimated.
		   * @param sac decides if another estimation iteration should be done.
		   * 
		   * @return the number of barrier sampling iterations performed.
		   */
		virtual
		size_t
		findBarriers(	LandscapeTopology* const lt
						, MinimumSelector* const ms
						, SamplingAbortionCriterion * const sac
						);
		
	protected:
		
		  //! the direct walk to use for the estimation of a barrier between
		  //! two minima
		const DirectWalk* const dw;
	};

} // namespace ell


#include "ell/Walk.hh"

namespace ell
{

	  /*!
	   * A barrier estimation approach using iterations of up-down-walks
	   * to estimate barriers between minima. An "up-down-walk" is a pair of
	   * walks that are run successively. The "up-walk" starts in a minimum of
	   * the landscape. The end of this walk is the starting point of the 
	   * following "down-walk" that is intended to reach a minimum of the
	   * landscape. Thus typically one will choose a WalkAdaptive or 
	   * WalkGradient as the "down-walk". The "up-walk" is intended to leave the
	   * minimum and to "climb up" (energetically) in the landscape to reach a
	   * good starting position for the down walk to find another minimum. If it
	   * is too low the down walk might end in the minimum the up walk started
	   * from and no barrier was found. Nevertheless this will count as an
	   * iteration of the BarrierSampling! 
	   * The minima to start from are choosen using the given minimum selector 
	   * and the sampling is aborted by the abortion criterion.
	   * 
	   * NOTE: The down-walk is not ensuring to end in a known minimum! Thus 
	   * new minima might be added to the topology according to the constructor
	   * setup! Even if new minima are ignored the iteration will be count.
	   * 
	   * This BarrierSampling MIGHT FIND new minima.
	   */
	class BS_UpDown : public BarrierSampling
	{
	public:
		
		  /*! Construction
		   * 
		   * @param upWalk the up-walk to perform starting from a selected 
		   *        minimum
		   * @param downWalk the down-walk to perform starting with the final
		   *        state visited by the up-walk. NOTE: this walk MUST END IN A
		   *        MINIMUM !!!
		   * @param addNewMinima whether or not new minima should be added to 
		   *        the lt or ignored.
		   */
		BS_UpDown(	const Walk* const upWalk
					, const Walk* const downWalk
					, const bool addNewMinima );
		
		  //! destruction
		virtual ~BS_UpDown();
		
		  /*!
		   * Performs an iteration of barrier estimations using successive
		   * up- and down-walks starting from a minimum of the landscape until 
		   * the sac decides that no further estimations are neccessary.
		   * 
		   * @param lt the landscape topology to add the barriers to and to
		   *        get the minima from between the barrier should be estimated.
		   * @param ms selects minima between which the next barrier should be
		   *        estimated.
		   * @param sac decides if another estimation iteration should be done.
		   * 
		   * @return the number of barrier sampling iterations performed.
		   */
		virtual
		size_t
		findBarriers(	LandscapeTopology* const lt
						, MinimumSelector* const ms
						, SamplingAbortionCriterion * const sac
						);
		
		
		
	protected:
		
		  //! the up walk to perform starting from a selected minimum
		const Walk* const upWalk;
		  //! the down walk to perform to find a minimum
		const Walk* const downWalk;
		  //! whether or not new minima should be ignored or added to the 
		  //! topology
		const bool addNewMinima;
	};

} // namespace ell




#endif /*BARRIERESTIMATION_HH_*/
