#ifndef S_EXPLICIT_HH_
#define S_EXPLICIT_HH_

#include "ell/State.hh"
#include "ell/IterableNeighbors.hh"

#include <biu/Alphabet.hh>
#include <biu/Graph_UD.hh>

namespace ell
{
	
	  /*! An explicit energy landscape representation using a graph 
	   * representation as the underlying data structure.
	   */ 
	class S_Explicit : public State, public IterableNeighbors
	{
	public:
		
		  /*! The landscapes topology defining the energy of the elements of
		   * the states (node label) and the neighboring of states (edges).
		   */
		class LandscapeGraph;
		
		
		  //! the specific ID string of this class
		static const std::string ID;
		
	protected:
		
		  //! the landscape graph representing the energy landscape
		const LandscapeGraph * lg;
		
		  //! the index of the node this state is representing
		size_t nodeIndex;
		
		  //! alphabet to compress and decompress a number string
		static const biu::Alphabet numberAlph;
		
		  //! length of all string representations of states of this landscape.
		size_t sRepLength;
		
	public:
		  /*! Constructs a state that represents the node of the given index
		   * of the explicite landscape representation. The neighbors of the
		   * node are defined by the adjacent nodes in the LandscapeGraph.
		   * 
		   * @param lg the landscape graph representing the energy landscape
		   * @param nodeIndex the index of the node this state is representing
		   */
		S_Explicit(	const LandscapeGraph* lg
					, const size_t nodeIndex );
		
		  //! Destruction
		virtual ~S_Explicit();
		
		////////////////////// STATE ITERFACE //////////////////////////////////
		
			/*!
			 * Access to a State subclass specific ID string to identify 
			 * instances of this class.
			 * @return the subclass specific ID string
			 */
		virtual const std::string& getID( void ) const;
	
		virtual bool			operator== (const State& state2) const;
		virtual bool			operator!= (const State& state2) const;
		
			/*! Implements a unique order on states based on their energy and
			 * string representation. A state is smaller than another one iff
			 * it has smaller energy or it has equal energy and a 
			 * lexicographically smaller string representation (tie breaker).
			 * 
			 * This function will be overwritten to achive a better performance
			 * than calling getEnergy() and getString().
			 * 
			 * If the energy function is non-degenerate the string comparison
			 * is obsolete.
			 * 
			 * @param state2 the State object to compare to
			 * @return true if this state is smaller than state2 according to 
			 *         the unique order of the states
			 */
		virtual bool			operator< (const State& state2) const;

			/*! Comparison function that compares two State pointer based on the
			 * less than operator '<' of the first State.
			 * The function can be used in STL algorithms, e.g. if only State
			 * pointer are stored but the ordering should be based on the 
			 * object order. 
			 * 
			 * @param s1 the State pointer of the object that is asked to be 
			 *           smaller (!=NULL)
			 * @param s2 the State pointer of the object that is asked to be 
			 *           bigger (!=NULL)
			 * @return s1->operator <(*s1);
			 */
		static
		bool less(const State* s1, const State* s2);
		
			/*! Returns the state specific energy. */
		virtual double		getEnergy() const;
		
			/*! Returns the minimal number of steps via valid
			 *  neighbored states from this to another valid State.
			 *  @param state2 the State to reach
			 */
		virtual unsigned int	getMinimalDistance(const State& state2) const;
		
			/*! Returns a pointer to a clone of the current state. 
			 * @param toFill a State to make a copy of this, or NULL if a new 
			 *        State should be created
			 * @return a pointer to a copy of this state, if toFill is != NULL
			 *         the return value corresponds to the updated state pointed
			 *         to by toFill, otherwise a new object is created !!!
			 * */
		virtual S_Explicit*	clone( State* toFill = NULL) const;
		
			/*! Returns a new State based on the current state. The new state
			 *  differs only by the information given by stringRep. */
		virtual State* fromString(const std::string& stringRep) const;
		
			/*! Returns a specific std::string representation of this
			 *  State.
			 */
		virtual std::string	toString() const;

			/*! Fill the given string with a specific std::string 
			 * representation of this State.
			 * @param toFill the string to overwrite
			 * @return the changed toFill in parameter
			 */
		virtual std::string& toString( std::string & toFill ) const;

		// neighborhood
			/*! Returns a virtual list of all VALID neighbored states in
			 *  the energy landscape in a specific order.
			 */
		virtual NeighborListPtr getNeighborList() const;

			/*! Returns a virtual list of all VALID neighbored states in
			 *  the energy landscape in a random order.
			 *  If a state is given, this is changed to a neighbor.
			 */
		virtual NeighborListPtr getRandomNeighborList() const;

			/*! Returns a VALID random neighbored state.
			 *  If a state is given, this is changed to a neighbor.
			 */
		virtual State* getRandomNeighbor(State* inPlaceNeigh = NULL) const;
		
		//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
		
			/*! Access to a compressed sequence representation of the state.
			 * @return the compressed sequence representation
			 */
		virtual CSequence compress(void) const;
		
			/*! Access to a compressed sequence representation of the state.
			 * @param toFill a data structure to write the compressed 
			 *               representation too
			 * @return the compressed sequence representation
			 */
		virtual CSequence& compress(CSequence& toFill) const;
		
			/*! Uncompresses a compressed sequencce representation into a new
			 * State object.
			 * @param cseq the compressed sequence representation of a state
			 * @param toFill a state object to uncompress too or NULL if a new 
			 *               object has to be created
			 * @return new State object that is encoded in cseq or NULL in error
			 *         case.
			 */
		virtual State* uncompress(const CSequence& cseq, State* toFill ) const;
		
			/*! Uncompresses a compressed sequencce representation into a this
			 * State object.
			 * @param cseq the compressed sequence representation of a state
			 * @return this or NULL in error case
			 */
		virtual State* uncompress(const CSequence& cseq);
		
		
		////////////////////// ITERABLE NEIGHBORS ITERFACE /////////////////////
		
		  //! How many neighbor indices are accessible.
		  //! @return the number of neighbors
		virtual 
		size_t 
		getNeighborNumber(void) const;
		
		  //! Access to a specific neighbor.
		  //! @param index the index of the neighbor in [0, getNeighborNumber())
		  //! @param neigh if != NULL this state should be converted into the 
		  //!              neighbor otherwise a new one is created and returned
		  //! @return the new neighbor state or NULL if the index is >= 
		  //! getNeighborNumber()
		virtual 
		State* 
		getNeighbor(const size_t index, State* neigh) const;
		
		  //! Undo the changes done to this state to generate the given neighbor.
		  //! The goal is to do less operations than doing a full copy like
		  //! (*neigh = *this).
		  //! @param index the neighbor index that was applied last 
		  //!              [ 0, getNeighborNumber() )
		  //! @param neigh the state resulting from the last changes
		  //! @return the copy of this object via undoing the last changes in 
		  //!         neigh or NULL if neigh == NULL
		virtual 
		State* 
		undoNeighborChange(const size_t index, State* const neigh) const;
		
		  //! Applies the necessary changes to a given state to get the neighbor.
		  //! Note: \@copy is expected to be a copy (i.e. *this == *copy) and
		  //! therefore direct changes are possible and != NULL. 
		  //! @param index the neighbor to generate
		  //! @param copy a copy of *this the changes should be applied to
		  //! @return copy = the neighbor state of the given index or NULL if the index
		  //!         is out of range
		virtual 
		State* 
		applyNeighborChange(const size_t index, State* const copy) const;

	};
	
	
	
	
	
	  /*! The explicit graph representation of the energy landscape where nodes
	   * are labeled with their energies.
	   */
	class S_Explicit::LandscapeGraph : public biu::Graph_UD
	{
	protected:
		  //! the energies of the nodes
		std::vector<double> nodeEnergy;
		
	public:
		  //! Construction
		LandscapeGraph( );
		
		  //! Destruction
		~LandscapeGraph();
		
		  //! adds a node with the given energy
		  //! @param energy of the node to add
		  //! @return the index of the newly added node
		size_t
		addNode( const double energy );
		
		  //! Access to the energy of a specified node
		  //! @param nodeIndex the index of the node to access
		  //! @return the energy of that node
		double
		getNodeEnergy( const size_t nodeIndex ) const;
		
	};

} // namespace ell

#endif /*S_EXPLICIT_HH_*/
