#ifndef STATE_H_
#define STATE_H_

#include <memory>
#include <string>
#include <limits.h>
#include <biu/VirtualList.hh>
#include <biu/Alphabet.hh>

	/** 
	 * The namespace ell is a shortcut for the Energy Landscape Library, 
	 * generated on the chair of Bioinformatics at the University of Freiburg.
	 * http://www.bioinf.uni-freiburg.de/
	 * 
	 * It covers a set of classes and methods to model and handle arbitrary 
	 * types of energy landscapes using the abstract interface class State 
	 * or derived subclasses. 
	 * 
	 * @author Martin Mann, Sebastian Will, Andreas Richter
	 */
namespace ell
{
	
	  //! A compressed string representation of a state object. The correct en-
	  //! or decoding is only possible using the same class's object. 
	typedef biu::Alphabet::CSequence CSequence;	
		/**
		 * An object of the class State represents a state in the energy
		 * landscape.
		 * It handles its fitness and neighborhood concerning the landscape.
		 * 
		 * A State is always valid in the subclass.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */

	class State
	{
	public:
		// TYPEDEFs
	
			//! Virtual list of neighbor states which can be accessed
			//! by a STL-like iterator.
		typedef biu::VirtualList<State> NeighborList;
		
			//! An autopointer to a NeighborList
		typedef std::auto_ptr<NeighborList> NeighborListPtr;

		State()
		{}
		
		virtual ~State()
		{}
		
			/*!
			 * Access to a State subclass specific ID string to identify 
			 * instances of this class.
			 * @return the subclass specific ID string
			 */
		virtual const std::string& getID( void ) const = 0;
		
		virtual bool			operator== (const State& state2) const = 0;
		virtual bool			operator!= (const State& state2) const = 0;
		
			/*! Implements a unique order on states based on their energy and
			 * string representation. A state is smaller than another one iff
			 * it has smaller energy or it has equal energy and a 
			 * lexicographically smaller string representation (tie breaker).
			 * 
			 * This function will be overwritten to achive a better performance
			 * than calling getEnergy() and getString().
			 * 
			 * If the energy function is non-degenerate the string comparison
			 * is obsolete.
			 * 
			 * @param state2 the State object to compare to
			 * @return true if this state is smaller than state2 according to 
			 *         the unique order of the states
			 */
		virtual bool			operator< (const State& state2) const;

			/*! Comparison function that compares two State pointer based on the
			 * less than operator '<' of the first State.
			 * The function can be used in STL algorithms, e.g. if only State
			 * pointer are stored but the ordering should be based on the 
			 * object order. 
			 * 
			 * @param s1 the State pointer of the object that is asked to be 
			 *           smaller (!=NULL)
			 * @param s2 the State pointer of the object that is asked to be 
			 *           bigger (!=NULL)
			 * @return s1->operator <(*s1);
			 */
		static
		bool less(const State* s1, const State* s2);
		
			/*! Returns the state specific energy. */
		virtual double		getEnergy() const = 0;
		
			/*! Returns the minimal number of steps via valid
			 *  neighbored states from this to another valid State.
			 *  @param state2 the State to reach
			 */
		virtual unsigned int	getMinimalDistance(const State& state2) const = 0;
		
			/*! Returns a pointer to a clone of the current state. 
			 * @param toFill a State to make a copy of this, or NULL if a new 
			 *        State should be created
			 * @return a pointer to a copy of this state, if toFill is != NULL
			 *         the return value corresponds to the updated state pointed
			 *         to by toFill, otherwise a new object is created !!!
			 * */
		virtual State*	clone( State* toFill = NULL) const = 0;
		
			/*! Returns a new State based on the current state. The new state
			 *  differs only by the information given by stringRep. */
		virtual State* fromString(const std::string& stringRep) const = 0;
		
			/*! Returns a specific std::string representation of this
			 *  State.
			 */
		virtual std::string	toString() const = 0;

			/*! Fill the given string with a specific std::string 
			 * representation of this State.
			 * @param toFill the string to overwrite
			 * @return the changed toFill in parameter
			 */
		virtual std::string& toString( std::string & toFill ) const = 0;

		// neighborhood
			/*! Returns a virtual list of all VALID neighbored states in
			 *  the energy landscape in a specific order.
			 */
		virtual NeighborListPtr getNeighborList() const = 0;

			/*! Returns a virtual list of all VALID neighbored states in
			 *  the energy landscape in a random order.
			 *  If a state is given, this is changed to a neighbor.
			 */
		virtual NeighborListPtr getRandomNeighborList() const = 0;

			/*! Returns a VALID random neighbored state.
			 *  If a state is given, this is changed to a neighbor.
			 */
		virtual State* getRandomNeighbor(State* inPlaceNeigh = NULL) const = 0;
		
		//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
		
			/*! Access to a compressed sequence representation of the state.
			 * @return the compressed sequence representation
			 */
		virtual CSequence compress(void) const = 0;
		
			/*! Access to a compressed sequence representation of the state.
			 * @param toFill a data structure to write the compressed 
			 *               representation too
			 * @return the compressed sequence representation
			 */
		virtual CSequence& compress(CSequence& toFill) const = 0;
		
			/*! Uncompresses a compressed sequencce representation into a new
			 * State object.
			 * @param cseq the compressed sequence representation of a state
			 * @param toFill a state object to uncompress too or NULL if a new 
			 *               object has to be created
			 * @return new State object that is encoded in cseq or NULL in error
			 *         case.
			 */
		virtual State* uncompress(const CSequence& cseq, State* toFill ) const = 0;
		
			/*! Uncompresses a compressed sequencce representation into a this
			 * State object.
			 * @param cseq the compressed sequence representation of a state
			 * @return this or NULL in error case
			 */
		virtual State* uncompress(const CSequence& cseq) = 0;
		
	};

}

#include <biu/assertbiu.hh>

namespace ell {

	inline
	bool
	State::
	operator< (const State& state2) const
	{
		  // check if same object
		if (this == &state2)
			return false;
		  // check if 
		  // - smaller energy or
		  // - equal energy and smaller string representation
		return ( this->getEnergy() < state2.getEnergy() )
				|| ( this->getEnergy() == state2.getEnergy()
					&& this->toString().compare(state2.toString()) < 0 );
	}
	
	inline
	bool
	State::
	less(const State* s1, const State* s2)
	{
		assertbiu( s1 != NULL, "no state s1 given (NULL)");
		assertbiu( s2 != NULL, "no state s2 given (NULL)");
		
		return s1->operator <( *s2 );
	}
}



#endif /*STATE_H_*/
