#ifndef WALK_HH_
#define WALK_HH_


#include "ell/State.hh"
#include "ell/NeighborGenerator.hh"
#include "ell/StateAcceptor.hh"
#include "ell/WalkAbortionCriterion.hh"
#include "ell/StateCollector.hh"

namespace ell {
	
	

	
	
	//!	Walk is an abstract class which defines:
	//! - a static function "walk" which takes all neccesary params for a generic walk
	//!		This static function Walk::walk is then invoked by
	//!	- a virtual member function "walk" which only takes a state to start and a walk collector
	//!  This virtual function then usually(not mandatory) invokes the static function Walk::walk 
	//!  with fixed params depending on the type of walk(=the class type), e.g. RandomWalk has 
	//!  member "length" and therefore creates a special WalkAbortionCriterion..
	//!  In addition every child of the Walk class has a static member which essentially does the same as 
	//!  the virtual member function(usually invoking Walk::walk) but with the use of additional params 
	//!  instead of members
	//!
	//!  @author: Hannes Kochni, Daniel Maticzka, Martin Mann
	class Walk {
	public:
		
		//! virtual destructor
			virtual ~Walk() {};
			
		//! pure virtual function, s is start state 
		//!
		//! @param start the begin of the walk
		//! @param scWalk the StateCollector each step of the walk is added to
		//! @param scRejected the StateCollector each rejected possible step is
		//!                   added to
		//! @return the modified scWalk StateCollector
		virtual 
			StateCollector*
			walk(	const State* const start, 
					StateCollector* const scWalk, 
					StateCollector* scRejected = NULL
				) const = 0;
		
		//! This function implements a generic walk starting from State start.
		//! The properties of this search are defined by the 
		//! implementations of NeighborhoodGenerator, StateAcceptor
		//! WalkAbortionCriterion and StateCollector.
		//!
		//! @param start the begin of the walk
		//! @param scWalk the StateCollector each step of the walk is added to
		//! @param ng the NeighborGenerator used to get the neighbor list of the
		//!           last walk State to generate the next possible steps
		//! @param sa the StateAcceptor that decides if a step is accepted or
		//!           rejected
		//! @param wac the walk abortion criterion that decides if the end of
		//!            the walk has been reached or a further step is to do
		//! @param scRejected the StateCollector each rejected possible step is
		//!                   added to
		//! @return the modified scWalk StateCollector
		static 
			StateCollector* 
			walk(	const State* const start, 
					StateCollector* const scWalk,
					const NeighborGenerator* const ng, 
					const StateAcceptor* const sa, 
					const WalkAbortionCriterion* const wac, 
					StateCollector* scRejected = NULL
				);
		
	protected:
	};
	
	
	
	

	//!	Implements an adaptive (decent) walk in the landscape, i.e.
	//! every state along the walk is smaller than its
	//! predecessor. Therefore, an adaptive walk ends in a local minimum.
	//! For each step successively all neighbors of the current walk end are
	//! checked if they are smaller according to the 'State::operator<(s2)'
	//! until a smaller one is found or all neighbors have been checked (end of
	//! the walk).
	//!
	//!	WalkAdaptive defines:
	//!  - an enum for the kind of adaptive walk
	//!  - a static function "walkAdaptive" which takes all neccesary params for an adaptive walk
	//!	 - a virtual member function "walk" which does the same as "walkAdaptive" but substitutes
	//!  the NeighborGentype of "walkAdaptive" with a member
	//!
	//!  @author: Hannes Kochni, Daniel Maticzka, Martin Mann
	class WalkAdaptive : public Walk {
	public:
		
		//! defines the kinds of Adaptive Walks: with random neighbor or succesive neighborgeneration
			enum NeighborGenType {SUCCESSIVE, RANDOM};
			
		//! constructor for a WalkAdaptive object, takes the type of walk
		//!
		//! @param ngt the type of the neighbor generator to use
		WalkAdaptive(	const NeighborGenType ngt
					);
		
		//! virtual destructor
		virtual ~WalkAdaptive() {};
		
		//! This static function implements an adaptive walk starting from State
		//! start by executing Walk::walk.
		//!
		//! @param start the begin of the walk
		//! @param scWalk the StateCollector each step of the walk is added to
		//! @param ngt the type of the neighbor generator to use
		//! @param scRejected the StateCollector each rejected possible step is
		//!                   added to
		//! @return the modified scWalk StateCollector
		static 
			StateCollector* 
			walkAdaptive(
				const State* const start, 
				StateCollector* const scWalk,
				const NeighborGenType ngt,
				StateCollector* const scRejected = NULL
				);

		// implements abstract function of superclass
		virtual
			StateCollector* 
			walk(
				const State* const start, 
				StateCollector* const scWalk,
				StateCollector* const scRejected = NULL
				) const;
				
	protected:
		//! WalkAdaptive object stores which kind neighbor generation is used 
		const NeighborGenType t;

	};

	
	
	
	//!	Implements a gradient (steepest decent) walk in the landscape, i.e.
	//! every state along the walk is the smallest of all neighbors of its
	//! predecessor. Therefore, a gradient walk ends in a local minimum.
	//! For each step all neighbors of the current walk end have to be checked
	//! if they are smaller according to the 'State::operator<(s2)'.
	//!
	//!  WalkGradient defines:
	//!  - a static function "walkGradient" which takes all neccesary params for an gradient walk
	//!	 - a virtual member function "walk" which does what "walkGradient" does
	//!
	//!  @author: Hannes Kochni, Daniel Maticzka, Martin Mann
	class WalkGradient : public Walk{
	public:
			
		//! constructs a gradient walk wrapper object
		WalkGradient();

		//! virtual destructor
		virtual ~WalkGradient() {};

		//! Static function to perform a gradient (steepest decent) walk.
		//!
		//! @param start the begin of the walk
		//! @param scWalk the StateCollector each step of the walk is added to
		//! @return the modified scWalk StateCollector
		static 
			StateCollector* 
			walkGradient(
				const State* const start, 
				StateCollector* const scWalk
				);
		
		// implementation of virtual member function for a gradient walk of an WalkGradient object
		virtual 
			StateCollector* 
			walk(
				const State* const start, 
				StateCollector* const scWalk,
				StateCollector* const scRejected
				) const;
		
	};

}

#include <biu/LimitedHash.hh>

namespace ell {
	
	/*! Implements a gradient (steepest decent) walk in the landscape, i.e.
	 * every state along the walk is the smallest of all neighbors of its
	 * predecessor. Therefore, a gradient walk ends in a local minimum.
	 * For each step all neighbors of the current walk end have to be checked
	 * if they are smaller according to the 'State::operator<(s2)'.
	 *
	 * This implementation utilizes a biu::LimitedHash to reduce the computation
	 * time. For each gradient neighbor, the final local minimum of the walk is
	 * stored. When calculating the gradient neighbor of a State, first a lookup
	 * in the hash is done, if the gradient local minimum is not already known
	 * from a previous calculation.
	 * 
	 * NOTE : This procedure is not able to generate the gradient path! Thus,
	 * use GradientWalk when interested in this!
	 *
	 * WalkGradient defines:
	 *  - a static function "walkGradient" which takes all neccesary params for an gradient walk
	 *  - a virtual member function "walk" which does what "walkGradient" does
	 *  - "setHashSize" which defines the maximal allowed elements in the hash  
	 *  - "getHashSize" which returns the maximal allowed elements in the hash  
	 *  - "clearHash" which removes all elements from the hash  
	 *
	 *  @author: Martin Mann
	 */
	class WalkGradientHashed : public Walk {
	public:
		
		typedef biu::LimitedHash< std::string, std::string > State2MinHash;
		
	protected:
		
		  //! hash of State string representation to the string representation
		  //! of the minimum it is mapped on
		mutable State2MinHash state2min;
		
	public:
			
		//! constructs a gradient walk wrapper object
		//! @param maxHashSize the maximal number of States to store in the hash
		WalkGradientHashed( const size_t maxHashSize = 10000 );

		//! virtual destructor
		virtual ~WalkGradientHashed() {};

		//! Static function to perform a gradient (steepest decent) walk.
		//!
		//! NOTE : If the hash lookup is successful not all states along the
		//! path are reported to the StateCollector! Only begin and end are
		//! ensured!
		//!
		//! @param start the begin of the walk
		//! @param scWalk the StateCollector each step of the walk is added to
		//! @return the modified scWalk StateCollector
		static 
			StateCollector* 
			walkGradient(
				const State* const start, 
				StateCollector* const scWalk,
				State2MinHash& state2min
				);
		
		//! Member function to perform a gradient (steepest decent) walk.
		//!
		//! NOTE : If the hash lookup is successful not all states along the
		//! path are reported to the StateCollector! Only begin and end are
		//! ensured!
		//!
		//! @param start the begin of the walk
		//! @param scWalk the StateCollector each step of the walk is added to
		//! @param scRejected is not used! can be NULL!
		//! @return the modified scWalk StateCollector
		virtual 
			StateCollector* 
			walk(
				const State* const start, 
				StateCollector* const scWalk,
				StateCollector* const scRejected
				) const;
		
		//! Sets the maximal number of elements allowed to store in the hash.
		//! @param s the maximal hash size
		virtual 
			void 
			setHashSize( const size_t s );

		//! Returns the maximal number of elements allowed to store in the hash.
		//! @return the maximal hash size
		virtual 
			const size_t
			getHashSize( void );
		
		//! Removes all elements currently stored in the hash.
		virtual 
			void
			clearHash( void );
		
	};

	
	

	//!	WalkMC is a class for a Monte Carlo Walk, it defines:
	//!  - a static function "walkMC" which takes all neccesary params for an monte carlo walk
	//!	 - a virtual member function "walk" which does the same as "walkMC" but substitutes
	//!  parameters WalkAbortionCriterion and beta of "walkGradient" with members
	//!
	//!  @author: Hannes Kochni, Daniel Maticzka, Martin Mann
	class WalkMC : public Walk{
	public:
		
		//! constructor which takes an AbortionCriterion and a term beta(1/k*T) TODO (formula correct?) 
		//! which is used by the StateAcceptor(based on Metropolitan criterion)
		WalkMC(	const WalkAbortionCriterion* const wac, 
					const double beta );
		
		//! virtual destructor		
		virtual ~WalkMC() {delete wac;};
		
		//! Static function for a Monte Carlo walk using the Metropolis
		//! criterion parametrized by kT. This is done utilizing a SA_Metropolis
		//! state acceptor object based on the Boltzmann weight of the energy
		//! difference between States.
		//!
		//! @param start the begin of the walk
		//! @param scWalk the StateCollector each step of the walk is added to
		//! @param wac the walk abortion criterion that decides if the end of
		//!            the walk has been reached or a further step is to do
		//! @param kT kT parameter of metropolis criterion e^(-E(State)/k*T))		
		//! @param scRejected the StateCollector each rejected possible step is
		//!                   added to
		//! @return the modified scWalk StateCollector
		static 
			StateCollector* 
			walkMC(	const State* const start, 
					StateCollector* const scWalk, 
					const WalkAbortionCriterion* const wac, 
					const double kT,
					StateCollector* const scRejected = NULL
					);
		
		// implementation of virtual member function for a monte carlo walk of an WalkMC object
		virtual 
			StateCollector* 
			walk(
				const State* const start, 
				StateCollector* const scWalk,
				StateCollector* const scRejected = NULL
			) const;
				
	protected:
		
		//! WalkMC object stores the WalkAbortionCriterion used by the virtual walk function
		const WalkAbortionCriterion* const wac;	
			
		//! WalkMC object stores kT used by the virtual walk function to create
		//! a SA_Metropolis state acceptor object.
		const double kT;
	};
	
	
	
	
	

	//!	WalkRandom is a class for a random walk, it defines:
	//!  - a static function "walkRandom" which takes all neccesary params for a random walk
	//!	 - a virtual member function "walk" which does the same as "walkRandom" but substitutes
	//!  the parameter wac of "walkRandom" with a member (see constructor)
	//!
	//!  @author: Hannes Kochni, Daniel Maticzka, Martin Mann
	class WalkRandom : public Walk {
	public:
		
		//! constructor which takes the length of the random walk 	
		//! @param wac the WalkAbortionCriterion that will decide if one more 
		//!           step is done or not
			WalkRandom(	
						const WalkAbortionCriterion* const wac
					);
			
		//! virtual destructor		
			virtual ~WalkRandom() {};
			
		//! static function for a random walk that uses a SA_All StateAcceptor
			//! @param start the begin of the walk
			//! @param scWalk the StateCollector each step of the walk is added to
			//! @param wac the WalkAbortionCriterion that will decide if one more 
			//!           step is done or not
			//! @param scRejected the StateCollector each rejected possible step is
			//!                   added to
			//! @return the modified scWalk StateCollector
		static 
			StateCollector* 
			walkRandom( const State* const start, 
						StateCollector* const scWalk,
						const WalkAbortionCriterion* const wac, 
						StateCollector* const scRejected = NULL
						);
		
		// implementation of virtual member function for a random walk of a WalkRandom object
		virtual 
			StateCollector* 
			walk(	const State* const start, 
					StateCollector* const scWalk,
					StateCollector* const scRejected = NULL
					) const;
		
	protected:
		
		//! the WalkAbortionCriterion that decides if one more step is done or not
		const WalkAbortionCriterion* const wac;
		
	};

	
	
} // end namespace ell

#endif /*WALK_HH_*/
