#ifndef LATTICEPROTEINMOVESTATE_HH_
#define LATTICEPROTEINMOVESTATE_HH_

#include "ell/State.hh"
#include <ell/IterableNeighbors.hh>
#include <biu/LatticeProtein_I.hh>
#include <biu/LatticeMoveSet.hh>

namespace ell
{

class S_LP : public State, public IterableNeighbors 
{
protected:

	biu::LatticeProtein_I* latProt;
	biu::LatticeMoveSet* moveSet;
	
public:
	
	  //! Each move is defined by a value of type Move.
	typedef size_t Move;
	
	  //! Constructs a lattice protein specific State.
	  //! @param latProt 	the lattice protein used by this State
	  //! @param moveSet	the move set used by this State
	S_LP(biu::LatticeProtein_I* latProt, biu::LatticeMoveSet* moveSet);
	S_LP(const S_LP& state);
	virtual ~S_LP();
	
	/*! Returns a pointer to a clone of the current state. 
	 * @param toFill a State to make a copy of this, or NULL if a new 
	 *        State should be created
	 * @return a pointer to a copy of this state, if toFill is != NULL
	 *         the return value corresponds to the updated state pointed
	 *         to by toFill, otherwise a new object is created !!!
	 * */
	virtual S_LP* clone(State* toFill = NULL) const = 0;
	
	  /*! Returns a new State based on the current state. The new state
	   *  differs only by the information given by stringRep. */
	virtual State* fromString(const std::string& stringRep) const = 0;		
	
	/*
	 * Functions using only latProt:
	 */
	virtual double getEnergy() const;
	virtual std::string	toString() const;
	
		/*! Fills the given string with a specific std::string 
		 * representation of this State.
		 * @param toFill the string to overwrite
		 * @return the changed toFill in parameter
		 */
	virtual std::string& toString( std::string & toFill ) const;
	
	
	// standard implementation: relay to LatticeProtein operators
	virtual bool operator== (const State& state2) const;
	virtual bool operator!= (const State& state2) const;
	
		/*! Implements a unique order on S_LP states based on their energy and
		 * absolute move string representation. A state is smaller than another 
		 * one iff it has smaller energy or it has equal energy and a 
		 * lexicographically smaller absolute move string representation 
		 * (tie breaker).
		 * 
		 * @param lp2 the S_LP object to compare to
		 * @return true if this state is smaller than lp2 according to 
		 *         the unique order of the states
		 */
	virtual bool operator< (const State& lp2) const;

	virtual NeighborListPtr getNeighborList() const;
	virtual NeighborListPtr getRandomNeighborList() const;
	
	virtual State* getRandomNeighbor(State* inPlaceNeigh = NULL) const;
	
	//////////////// ITERABLE_NEIGHBORS ///////////////////////////////
	
	  //! How many neighbor indices are accessible.
	  //! @return the number of neighbors
	virtual size_t getNeighborNumber(void) const;
	
	  //! Access to a specific neighbor.
	  //! @param index the index of the neighbor in [0, getNeighborNumber())
	  //! @param neigh if != NULL this state should be converted into the 
	  //!              neighbor otherwise a new one is created and returned
	  //! @return the new neighbor state or NULL if the index is >= 
	  //! getNeighborNumber()
	virtual State* getNeighbor(const size_t index, State* neigh) const;
	
	  //! Undo the changes done to this state to generate the given neighbor.
	  //! The goal is to do less operations than doing a full copy like
	  //! (*neigh = *this).
	  //! @param index the neighbor index that was applied last 
	  //!              [ 0, getNeighborNumber() )
	  //! @param neigh the state resulting from the last changes ( != NULL)
	  //! @return the copy of this object via undoing the last changes in 
	  //!         neigh
	virtual State* undoNeighborChange(const size_t index, State* const neigh) const;
	
	  //! Applies the necessary changes to a given state to get the neighbor.
	  //! Note: neigh is expected to be a copy (i.e. *this == *neigh) and
	  //! therefore direct changes are possible or to be NULL. If 
	  //! (neigh==NULL) a copy of this has to be generated, modified, and
	  //! returned.
	  //! @param index the neighbor to generate
	  //! @param neigh a copy of this the changes should be applied to
	  //! @return the neighbor state of the given index or NULL if the index is
	  //!         out of range
	virtual State* applyNeighborChange(const size_t index, State* const neigh) const;
    
    	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
		/*! Access to a compressed sequence representation of the state.
		 * @return the compressed sequence representation
		 */
	virtual CSequence compress(void) const = 0;
	
		/*! Access to a compressed sequence representation of the state.
		 * @param toFill a data structure to write the compressed 
		 *               representation too
		 * @return the compressed sequence representation
		 */
	virtual CSequence& compress(CSequence& toFill) const = 0;
	
		/*! Uncompresses a compressed sequencce representation into a new
		 * State object.
		 * @param cseq the compressed sequence representation of a state
		 * @param toFill a state object to uncompress too or NULL if a new 
		 *               object has to be created
		 * @return new State object that is encoded in cseq or NULL in error
		 *         case.
		 */
	virtual State* uncompress(const CSequence& cseq, State* toFill = NULL) const = 0;
	
		/*! Uncompresses a compressed sequence representation into a this
		 * State object.
		 * @param cseq the compressed sequence representation of a state
		 * @return this or NULL in error case
		 */
	virtual State* uncompress(const CSequence& cseq) = 0;
};

} // namespace ell

#endif /*LATTICEPROTEINMOVESTATE_HH_*/
