#ifndef RNAFREEENERGY_TB_HH_
#define RNAFREEENERGY_TB_HH_


#include <biu/RNAStructure_TB.hh>

namespace ell
{

		/**
		 * This subclass of RNAStructure implements the free energy as
		 * the underlying energy function with the constant alphabet ACGU
		 * and the constant valid RNA base pairs AU, CG and GU (and their
		 * reverse).
		 * 
		 * @author Martin Mann, Sebastian Will
		 * @author Salem Dekelbab
		 */
	class RNAFreeEnergy_TB : public biu::RNAStructure_TB
	{
	protected:
			/*! The common BasePair object of all RNAFreeEnergy_TB objects.
			 *  The valid RNA base pairs are AU, CG und GU and their reverse.
			 */
		static const biu::AllowedBasePairs		COMMON_BPAIRS;

			/*! The common Alphabet object of all RNAFreeEnergy_TB objects.
			 *  It contains the elements A, C, G and U.
			 */
		static const biu::Alphabet				COMMON_ALPHABET;
		
		  //! dummy value for energy to inform that recalculation is necessary
		static const double ENERGY_INF; 
		
		  //! free energy of the molecule calculated using the Vienna RNA package
		mutable double energy;
		
		short* seqVienna;
		short* pairTable;

	public:
		
		  /*! whether or not the structure should calculate the energy 
		   * difference if the structure is changed via moves or not
		   */ 
		static bool calculateEnergyChanges;
		
			/*! A single move (open or close one bond) on a RNA structure
		 	 *  at positions SingleMove.first and SingleMove.second.
		 	 */
		typedef std::pair<size_t, size_t> SingleMove;

		RNAFreeEnergy_TB(	const std::string& rnaSeqStr
							, const std::string& rnaStructBracketDotStr
							, const double energy = ENERGY_INF );

		RNAFreeEnergy_TB(	biu::Sequence* rnaSeq
							, const biu::Structure* const rnaStructBracketDot
							, const bool seqIsShared = false
							, const double energy = ENERGY_INF );

		RNAFreeEnergy_TB(const RNAFreeEnergy_TB& rnaFreeEnerg);
		
		virtual ~RNAFreeEnergy_TB();
		
			/*! Applies inplace a single move at the positions
			 *  given by move.
			 *  In case of invalid moves, the behaviour
			 *  of the program is not specified.
			 */
		void applySingleMoveInPlace(const SingleMove& move);
		
			/*! Returns whether a move produces a valid RNA structure.
			 *  A move can only be valid if the first move position
			 *  move.first is less than the second move position
			 *  move.second!
			 */
		bool isValidSingleMove(const SingleMove& move) const;
		
		static const biu::Alphabet* getSequenceAlphabet() {
			return &COMMON_ALPHABET;
		}

	// abstract functions (BioMolecule)
		
			/*! Returns the free energy of this RNA structure
			 *  allowing AU, CG and GU base base pairs.
			 *  The energy is calculated via the Vienna RNA package functions.
			 * NOTE : The value UNIT is given ( kcal / mol ) !!!
			 */
		virtual double	getEnergy( void ) const;
		
		/*!
		 * Sets the energy of this RNA structure such that it is not 
		 * calculated via the Vienna RNA package.
		 * 
		 * NOTE: It has to be the same energy as calculated by the 
		 *       Vienna RNA 'energy_of_struct' function, otherwise 
		 *       assertions will be broken.
		 * 
		 * NOTE : The value UNIT is given ( kcal / mol ) !!!
		 * 
		 * @param energy the energy of this structure
		 */
	virtual void setEnergy( const double energy );

		RNAFreeEnergy_TB&	operator= (const RNAFreeEnergy_TB& rnaStruct2);

		//! Sets the RNA structure in bracket notation.
		//! @param str the structure to set in bracket notation of correct 
		//!         length
	virtual void setStructure(const biu::Structure& str);

	protected:
		
		  //! Informs the molecule that the sequence or structure has been 
		  //! changed to force recalculation of lazy properties (e.g. energy)
		virtual void moleculeChanged();
		
		  /*!
		   * Updates seqVienna to contain the structure encoding in Vienna RNA
		   * style.
		   */
		virtual void updateSeqVienna();
		
	protected:
		
		  //! whether or not the fold parameters of the Vienna package are 
		  //! already set or not
		static bool ViennaFoldParamsSet;
		
		  //! sets the Vienna package fold parameters and sets the variable
		  //! ViennaFoldParamsSet to TRUE.
		static void setViennaFoldParams(void);
		
	};

} // namespace biu

  // include definitions
#include "ell/rna/RNAFreeEnergy_TB.icc"

#endif /*RNAFREEENERGY_TB_HH_*/
