#ifndef RNAFREEENERGYSINGLEMOVESTATE_HH_
#define RNAFREEENERGYSINGLEMOVESTATE_HH_


#include "ell/State.hh"
#include "ell/rna/RNAFreeEnergy.hh"

namespace ell
{
		/**
		 * This class represents an RNA structure in the energy landscape.
		 * Neighbors are generated by adding or removing one bond to the RNA 
		 * structure known as single moves.
		 * 
		 * The free energy calculated using Vienna RNA package functionality.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class S_RNAfe_SingleM : public State
	{
	protected:	
		typedef RNAFreeEnergy::SingleMove	SingleMove;

		class NeighborList : public State::NeighborList
		{
		protected:
				//! the state of origin of the neighbors
			const S_RNAfe_SingleM* const origin;

		public:
				//! State of an iterator. Usually the information in ItState is
				//! used for computing the next list entry 
			class ItState : public State::NeighborList::ItState
			{
			public:
					//! the last applied single move
				SingleMove	lastMove;

				ItState() : lastMove() {}
				ItState(const ItState &itstate)
				 : lastMove(itstate.lastMove) {}
				virtual ~ItState() {}

				virtual ItState* clone() const {
					return new ItState(*this);
				}
			};

			NeighborList(const S_RNAfe_SingleM* _origin);
			
			virtual ~NeighborList();

				/*! Returns a pointer to the first element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

		class RandomNeighborList : public State::NeighborList
		{
		public:
			class ItState;
			
		protected:
				//! the state of origin of the neighbors
			const S_RNAfe_SingleM* const origin;

				//! Changes the mode of choice of the next single move from
				//! choice out of all single moves to choice out of all
				//! unchosen valid single moves
			void	switchMode(ItState* itstate) const;
			
		public:
			class ItState : public State::NeighborList::ItState
			{
			public:
					//! if the numbers of valid single moves that have been
					//! selected more than once exceeds this threshold, the
					//! mode of choice will be changed to choice out of all
					//! unchosen valid moves
				static const size_t SWITCH_MODE_THRESHOLD;
				
					//! counts down from SWITCH_MODE_THRESHOLD to zero for
					//! every valid move chosen more than once
				size_t switchModeValue;
				
					//! set containing all valid single moves that were randomly
					//! chosen during iteration
				std::set<SingleMove> chosenMoves;
				
					//! vector containing all VALID single moves which have
					//! not been chosen yet
				std::vector<SingleMove> unchosenValidMoves;

				ItState() : switchModeValue(SWITCH_MODE_THRESHOLD),
							chosenMoves(),
							unchosenValidMoves()
				{}
				
				ItState(const ItState &itstate) 
					: switchModeValue(itstate.switchModeValue),
					  chosenMoves(itstate.chosenMoves),
					  unchosenValidMoves(itstate.unchosenValidMoves)
				{}
				
				virtual ~ItState() 
				{}

				virtual ItState* clone() const {
					return new ItState(*this);
				}
			};

			RandomNeighborList(const S_RNAfe_SingleM* _origin);
			
			virtual ~RandomNeighborList();

				/*! Returns a pointer to a random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

		RNAFreeEnergy rnaFreeEnergy;

		biu::Alphabet::AlphElem open;
		biu::Alphabet::AlphElem none;
		biu::Alphabet::AlphElem close;

			/*! Returns the first VALID single move.
			 *  Returns SingleMove(0,RNAStructure::INVALID_INDEX)
			 *  if no first valid single move exits.
			 */
		SingleMove	firstValidSingleMove() const;
		
			/*! Returns the next VALID single move if lastSingleMove
			 *  was the last single move.
			 *  Returns SingleMove(0,RNAStructure::INVALID_INDEX)
			 *  if no next valid single move exits.
			 *
			 *  lastSingleMove has to be an valid move!
			 */
		SingleMove nextValidSingleMove(const SingleMove lastSingleMove) const;
		
			/*! Searches for the next valid single move and starts
			 *  the search for the first move position at pos 1
			 *  and for the second move position at pos2.
			 *  If a move was found, it will be returned, otherwise
			 *  SingleMove(0,RNAStructure::INVALID_INDEX) will be returned.
			 */
		SingleMove searchForNextValidSingleMove(size_t pos1,
				 							size_t pos2) const;
	
	public:
		
		 //! the specific ID string of this class
		static const std::string ID;
		
		S_RNAfe_SingleM(	const std::string& rnaSeqStr,
							const std::string& rnaStructBracketNotStr);
		
		S_RNAfe_SingleM(	const RNAFreeEnergy& rna );
				
		  //! copy construction
		  //! @param toCopy the state to copy
		S_RNAfe_SingleM(	const S_RNAfe_SingleM& toCopy);
						
		virtual ~S_RNAfe_SingleM();
		
			/*!
			 * Access to a State subclass specific ID string to identify 
			 * instances of this class.
			 * @return the subclass specific ID string
			 */
		virtual const std::string& getID( void ) const;


	// abstract functions (State)
		virtual bool	operator== (const State& state2) const;
		virtual bool	operator!= (const State& state2) const;
			
			/*! Implements a unique order on RNA states based on their energy 
			 * and string representation. An RNA is smaller than another one 
			 * iff it has smaller energy or it has equal energy and a 
			 * lexicographically smaller string representation (tie breaker).
			 * 
			 * @param rna2 the RNA State object to compare to
			 * @return true if this state is smaller than rna2 according to 
			 *         the unique order of the states
			 */
		virtual bool	operator< (const State& rna2) const;

		
			//! Returns the state specific energy.
			//! The energy is calculated via the Vienna RNA package functions.
			//! NOTE : The value UNIT is given ( kcal / mol ) !!!
		virtual double		getEnergy() const;
		
			/*! Returns the minimal number of steps via valid
			 *  neighbored states from this to another valid State.
			 *  @param state2 the State to reach
			 */
		virtual unsigned int	getMinimalDistance(const State& state2) const;
		
			/*! Returns a pointer to a clone of the current state. 
			 * @param toFill a State to make a copy of this, or NULL if a new 
			 *        State should be created
			 * @return a pointer to a copy of this state, if toFill is != NULL
			 *         the return value corresponds to the updated state pointed
			 *         to by toFill, otherwise a new object is created !!!
			 * */
		virtual S_RNAfe_SingleM* clone(State* toFill = NULL) const;
		
			/*! Returns a new State based on the current state. The new state
			 *  differs only by the information given by stringRep. */
		virtual State* fromString(const std::string& stringRep) const;			
		
			/*! Returns a specific std::string representation of this
			 *  State.
			 * 
			 * @return the dot-bracket-notation of the structure */
		virtual std::string toString() const;
		
			/*! Fills the given string with a specific std::string 
			 * representation of this State.
			 * @param toFill the string to overwrite
			 * @return the changed toFill in parameter
			 */
		virtual std::string& toString( std::string & toFill ) const;
		
		
			/*! Access to the nucleotide sequence of this RNA molecule.
			 * @return the RNA sequence */ 
		virtual std::string getSequence() const;
		
			/*! Access to the structure of this RNA molecule encoded in 
			 * dot-bracket notation.
			 * @return the RNA structure */ 
		virtual std::string getStructure() const;

		virtual NeighborListPtr getNeighborList() const;
		
		virtual NeighborListPtr getRandomNeighborList() const;
		
		virtual State*	getRandomNeighbor(State* inPlaceNeigh = NULL) const;

		//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
		
			/*! Access to a compressed sequence representation of the state.
			 * @return the compressed sequence representation
			 */
		virtual CSequence compress(void) const ;
		
			/*! Access to a compressed sequence representation of the state.
			 * @param toFill a data structure to write the compressed 
			 *               representation too
			 * @return the compressed sequence representation
			 */
		virtual CSequence& compress(CSequence& toFill) const;
		
			/*! Uncompresses a compressed sequencce representation into a new
			 * State object.
			 * @param cseq the compressed sequence representation of a state
			 * @param toFill a state object to uncompress too or NULL if a new 
			 *               object has to be created
			 * @return new State object that is encoded in cseq or NULL in error
			 *         case.
			 */
		virtual State* uncompress(const CSequence& cseq, State* toFill) const;
		
			/*! Uncompresses a compressed sequencce representation into a this
			 * State object.
			 * @param cseq the compressed sequence representation of a state
			 * @return this or NULL in error case
			 */
		virtual State* uncompress(const CSequence& cseq);
		
		
	};

} // namespace ell

#endif /*RNAFREEENERGYSINGLEMOVESTATE_HH_*/
