#include "ell/protein/S_LP_PivotM.hh"

#include <iostream>
#include <biu/LatticeDescriptorCUB.hh>
#include <biu/LatticeModel.hh>
#include <biu/LatticeProtein_Ipnt.hh>
#include <biu/PullMoveSet.hh>
#include <ell/NeighborGenerator.hh>
#include <ell/StateAcceptor.hh>
#include <ell/WalkAbortionCriterion.hh>
#include <ell/StateCollector.hh>
#include <ell/Walk.hh>
#include <ell/State.hh>
#include <biu/RandomNumberGenerator.hh>
#include <biu/RandomNumberFactory.hh>


using namespace ell;

void testWalk(bool seqShared = true) {
	std::cout	<<"============================\n"
				<<" testing walk\n"
				<<"============================" <<std::endl;


	/*
	 * Building Lattice related objects.
	 */
	biu::LatticeDescriptorCUB ld;
	biu::LatticeModel lm(&ld);
	biu::Alphabet alph("HP",1);
	biu::EnergyMatrix E = biu::EnergyMatrix(
			alph.getAlphabetSize(),
			alph.getAlphabetSize(),
			0.0);
	E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	biu::ContactEnergyFunction f(&alph, &E, &lm);
	
	/*
	 * Building Protein related objects.
	 */
	std::string seqStr = "PHPHHHH";
	biu::Sequence seq = alph.getSequence(seqStr);
	std::string absMoveStr = "UUUUUU";
	bool isAbsMove = true;
	biu::LatticeProtein_I* latProt = new biu::LatticeProtein_Ipnt
											(&lm,&f,&seq,seqShared,absMoveStr,
											isAbsMove);
	biu::LatticeMoveSet* moveSet = new biu::PullMoveSet(&lm);
	
	/*
	 * Building State
	 */
	std::cout	<< "S_LP_PivotM* s = new S_LP_PivotM(&lm, &f, latProt, moveSet)"
					<< std::endl;
	State* s = new S_LP_PivotM(latProt, moveSet);
	std::cout	<< "=> s->getID() = " <<s->getID() << std::endl;

	/*
	 * Testing public functions
	 */
	std::cout 	<< "s->getEnergy() = " 
				<< s->getEnergy()
				<< std::endl;
	
	std::cout 	<< "s->toString() = " 
				<< s->toString()
				<< std::endl;
	
	std::cout	<< "creating State* s2"
				<< std::endl;
	State* s2 = new S_LP_PivotM(latProt->clone(), moveSet->clone());
	
	std::cout	<< "(*s == *s2) = "
				<< (*s == *s2) 
				<< std::endl;
	
	std::cout	<< "(*s != *s2) = "
				<< (*s != *s2)
				<< std::endl;
	
	{
		std::cout	<< "--> s->clone(s2)" <<std::endl;
		s->clone(s2);
	
		std::cout	<< "(*s == *s2) = "
					<< (*s == *s2) 
					<< std::endl;
		
		std::cout	<< "(*s != *s2) = "
					<< (*s != *s2)
					<< std::endl;
	}
	
	/* 
	 * Testing walk
	 */
	 
	NeighborGenerator* ng = new NG_Successive();
	StateAcceptor* sa = new SA_E_LeEq();
	WalkAbortionCriterion* wac = new WAC_MaxLength(5);
	SC_Outstream* sc = new SC_Outstream(std::cout," a\n");
	
	Walk::walk(s, sc, ng, sa, wac);
		
	std::cout 	<< "Number Of Accepted States: " 
				<< sc->size()
				<< std::endl;
		
		
	delete ng;
	delete sa;
	delete wac;
	delete sc;
	delete s;
	delete s2;
}

void testGetRandomNeighbor(bool seqShared = true) {
	std::cout	<<"============================\n"
				<<" testing getRandomNeighbor\n"
				<<"============================" <<std::endl;

	/*
	 * Building Lattice related objects.
	 */
	biu::LatticeDescriptorCUB ld;
	biu::LatticeModel lm(&ld);
	biu::Alphabet alph("HP",1);
	biu::EnergyMatrix E = biu::EnergyMatrix(
			alph.getAlphabetSize(),
			alph.getAlphabetSize(),
			0.0);
	E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	biu::ContactEnergyFunction f(&alph, &E, &lm);
	
	/*
	 * Building Protein related objects.
	 */
	std::string seqStr = "PHPHHHH";
	biu::Sequence seq = alph.getSequence(seqStr);
	std::string absMoveStr = "FFFFFF";
	bool isAbsMove = true;
	biu::LatticeProtein_I* latProt = new biu::LatticeProtein_Ipnt
											(&lm,&f,&seq,seqShared,absMoveStr,
											isAbsMove);
	biu::LatticeMoveSet* moveSet = new biu::PullMoveSet(&lm);
	
	/*
	 * Building State
	 */
	std::cout	<< "S_LP_PivotM* s = new S_LP_PivotM(&lm, &f, latProt, moveSet)"
					<< std::endl;
	S_LP* s = new S_LP_PivotM(latProt, moveSet);

	std::cout 	<< "s->toString() = "
				<< s->toString() << std::endl;
	std::cout	<< "s2 = s->getRandomNeighbor()" << std::endl;
	State* s2 = s->getRandomNeighbor();
	std::cout 	<< "s2->toString() = "
				<< s2->toString()  << std::endl;
	
	delete s;
	delete s2;
}

void testCompressedSequence(bool seqShared = true) {
	std::cout	<<"============================\n"
				<<" testing compressed sequence\n"
				<<"============================" <<std::endl;
	
	/*
	 * Building Lattice related objects.
	 */
	biu::LatticeDescriptorCUB ld;
	biu::LatticeModel lm(&ld);
	biu::Alphabet alph("HP",1);
	biu::EnergyMatrix E = biu::EnergyMatrix(
			alph.getAlphabetSize(),
			alph.getAlphabetSize(),
			0.0);
	E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	biu::ContactEnergyFunction f(&alph, &E, &lm);
	
	/*
	 * Building Protein related objects.
	 */
	std::string seqStr = "PHPHHHH";
	biu::Sequence seq = alph.getSequence(seqStr);
	std::string absMoveStr = "FFFFFF";
	std::string absMoveStr2 = "FRFFFF";
	bool isAbsMove = true;
	biu::LatticeProtein_I* latProt = new biu::LatticeProtein_Ipnt
											(&lm,&f,&seq,seqShared,absMoveStr,
											isAbsMove);
	biu::LatticeProtein_I* latProt2 = new biu::LatticeProtein_Ipnt
											(&lm,&f,&seq,seqShared,absMoveStr2,
											isAbsMove);
	biu::LatticeMoveSet* moveSet = new biu::PullMoveSet(&lm);
	
	/*
	 * Building State
	 */
	std::cout	<< "S_LP_PivotM* s = new S_LP_PivotM(&lm, &f, latProt, moveSet)"
					<< std::endl;
	S_LP* s = new S_LP_PivotM(latProt->clone(), moveSet->clone());
	S_LP* s0 = new S_LP_PivotM(latProt->clone(), moveSet->clone());
	S_LP* s2 = new S_LP_PivotM(latProt2->clone(), moveSet->clone());
	S_LP* s3 = new S_LP_PivotM(latProt2->clone(), moveSet->clone());

	/*
	 * testing compress & uncompress functions.
	 */
	
    /*
     * testing compress(void) & uncompress(const CSequence& cseq)
     */
    std::cout   << std::endl 
                << "testing compress(void) & uncompress(const CSequence& cseq):"
                << std::endl;
    std::cout   << "s->toString() = " << s->toString() << std::endl;
    std::cout   << "cseq = s->compress()"  << std::endl;
    CSequence cseq = s->compress();
    std::cout   << "cseq.size() = " << cseq.size() << std::endl;
    std::cout   << "s2->toString() = " << s2->toString() << std::endl;
    std::cout   << "s2->uncompress(cseq)" << std::endl;
    s2->uncompress(cseq);
    std::cout   << "s2->toString() = " << s2->toString() << std::endl;
    
    
    /*
     * testing  compress(CSequence& toFill) &
     *          uncompress(const CSequence& cseq, State* toFill)
     */
    std::cout   << std::endl 
                << "testing compress(CSequence& cseq) & "
                << "uncompress(const CSequence& cseq, State* toFill):"
                << std::endl;
    std::cout   << "s3->toString() = " << s3->toString() << std::endl;
    std::cout   << "s3->compress(cseq)" << std::endl;
    s3->compress(cseq);
    std::cout   << "cseq.size() = " << cseq.size() << std::endl;
    std::cout	<< "=> uncompress to new State" << std::endl;
    std::cout	<< "State* s4 = NULL" << std::endl;
    State* s4 = NULL;
    std::cout   << "s4 = s3->uncompress(cseq, s4);" << std::endl;
    s4 = s3->uncompress(cseq, s4);
    std::cout   << "s4->toString() = " << s4->toString() << std::endl;
    std::cout	<< "=> uncompress to existing State" << std::endl;
    std::cout   << "s0->toString() = " << s0->toString() << std::endl;
    std::cout	<< "s0 = s3->uncompress(cseq, s0)" << std::endl;
    s0 = dynamic_cast<S_LP*>(s3->uncompress(cseq, s0));
    std::cout   << "s0->toString() = " << s0->toString() << std::endl;
    
    delete s;
    delete s0;
    delete s2;
    delete s3;
    delete s4;
    delete latProt;
    delete latProt2;
    delete moveSet;
}

void testGetNeighbor(bool seqShared = true) 
{
	std::cout	<<"============================\n"
				<<" testing getNeighbor\n"
				<<"============================" <<std::endl;
	/*
	 * Building Lattice related objects.
	 */
	biu::LatticeDescriptorCUB ld;
	biu::LatticeModel lm(&ld);
	biu::Alphabet alph("HP",1);
	biu::EnergyMatrix E = biu::EnergyMatrix(
			alph.getAlphabetSize(),
			alph.getAlphabetSize(),
			0.0);
	E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	biu::ContactEnergyFunction f(&alph, &E, &lm);
	
	/*
	 * Building Protein related objects.
	 */
	std::string seqStr = "PHPHHHH";
	biu::Sequence seq = alph.getSequence(seqStr);
	std::string absMoveStr = "FFFFFF";
	bool isAbsMove = true;
	biu::LatticeProtein_I* latProt = new biu::LatticeProtein_Ipnt
											(&lm,&f,&seq,seqShared,absMoveStr,
											isAbsMove);
	biu::LatticeMoveSet* moveSet = new biu::PullMoveSet(&lm);
	
	/*
	 * Building State
	 */
	std::cout	<< "S_LP_PivotM* s = new S_LP_PivotM(&lm, &f, latProt, moveSet)"
					<< std::endl;
	S_LP* s = new S_LP_PivotM(latProt, moveSet);

	std::cout 	<< "s->toString() = "
				<< s->toString() << std::endl;
	std::cout	<< "s2 = s->getNeighbor(36,NULL)" << std::endl;
	State* s2 = s->getNeighbor(36, NULL);
	std::cout 	<< "s2->toString() = "
				<< s2->toString()  << std::endl;
	
	delete s;
	delete s2;
}

void testSuccessiveNeighborList(bool seqShared = true) {
	std::cout	<<"============================\n"
				<<" testing SuccessiveNeigborList\n"
				<<"============================" <<std::endl;

	/*
	 * Building Lattice related objects.
	 */
	biu::LatticeDescriptorCUB ld;
	biu::LatticeModel lm(&ld);
	biu::Alphabet alph("HP",1);
	biu::EnergyMatrix E = biu::EnergyMatrix(
			alph.getAlphabetSize(),
			alph.getAlphabetSize(),
			0.0);
	E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	biu::ContactEnergyFunction f(&alph, &E, &lm);
	
	/*
	 * Building Protein related objects.
	 */
	std::string seqStr = "PHPHHHH";
	biu::Sequence seq = alph.getSequence(seqStr);
	std::string absMoveStr = "FFFFFF";
	bool isAbsMove = true;
	biu::LatticeProtein_I* latProt = new biu::LatticeProtein_Ipnt
											(&lm,&f,&seq,seqShared,absMoveStr,
											isAbsMove);
	biu::LatticeMoveSet* moveSet = new biu::PullMoveSet(&lm);
	
	/*
	 * Building State
	 */
	std::cout	<< "S_LP_PivotM* s = new S_LP_PivotM(&lm, &f, latProt, moveSet)"
					<< std::endl;
	S_LP* s = new S_LP_PivotM(latProt, moveSet);
	std::cout	<< "s->toString() = " << s->toString() << std::endl;
	
	/*
	 * Testing: SuccessiveNeighborList
	 */
	State::NeighborListPtr neighbors = s->getNeighborList();
	State::NeighborList::Iterator it = neighbors->begin();
	
	int i=1;
	while( it != neighbors->end()) {
		std::cout	<< i << ": (*it).toString() = " 
					<< (&(*it) == NULL? "NULL" : (*it).toString()) << std::endl;
		i++;
		++it; // next neigh
	}

	delete s;
}

void testRandomNeighborList(bool seqShared = true) {
	std::cout	<<"============================\n"
				<<" testing RandomNeighborList\n"
				<<"============================" <<std::endl;

	/*
	 * Building Lattice related objects.
	 */
	biu::LatticeDescriptorCUB ld;
	biu::LatticeModel lm(&ld);
	biu::Alphabet alph("HP",1);
	biu::EnergyMatrix E = biu::EnergyMatrix(
			alph.getAlphabetSize(),
			alph.getAlphabetSize(),
			0.0);
	E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	biu::ContactEnergyFunction f(&alph, &E, &lm);
	
	/*
	 * Building Protein related objects.
	 */
	std::string seqStr = "PHPHHHH";
	biu::Sequence seq = alph.getSequence(seqStr);
	std::string absMoveStr = "FFFFFF";
	bool isAbsMove = true;
	biu::LatticeProtein_I* latProt = new biu::LatticeProtein_Ipnt
											(&lm,&f,&seq,seqShared,absMoveStr,
											isAbsMove);
	biu::LatticeMoveSet* moveSet = new biu::PullMoveSet(&lm);
	
	/*
	 * Building State
	 */
	std::cout	<< "S_LP_PivotM* s = new S_LP_PivotM(&lm, &f, latProt, moveSet)"
					<< std::endl;
	S_LP* s = new S_LP_PivotM(latProt, moveSet);
	std::cout	<< "s->toString() = " << s->toString() << std::endl;
	
	/*
	 * Testing: RandomNeighborList
	 */
	State::NeighborListPtr neighbors = s->getRandomNeighborList();
	State::NeighborList::Iterator it = neighbors->begin();
	
	int i = 1;
	do {
		std::cout	<< i << ": (*it).toString() = " 
					<< (*it).toString() << std::endl;
		i++;
		++it;
	} while (it != neighbors->end());
	
	delete s;
}

int main(int argc, char** argv)
{
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : S_LP_PivotM\n"
				<<"===========================================" <<std::endl;

		// init randomizer	
	std::cout	<<"=> RND::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	const unsigned int seed = 1;
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);


	std::cout	<<"===========================================\n"
				<<"        WITH SHARED SEQUENCE\n"
				<<"===========================================" <<std::endl;
	bool seqShared = true;
	
	testGetRandomNeighbor(seqShared);
	testGetNeighbor(seqShared);

	testRandomNeighborList(seqShared);
	testSuccessiveNeighborList(seqShared);

	testWalk(seqShared);

	testCompressedSequence(seqShared);
	
	std::cout	<<"===========================================\n"
				<<"        WITH OWN SEQUENCE\n"
				<<"===========================================" <<std::endl;

	seqShared = false;
				
	testGetRandomNeighbor(seqShared);
	testGetNeighbor(seqShared);

	testRandomNeighborList(seqShared);
	testSuccessiveNeighborList(seqShared);

	testWalk(seqShared);

	testCompressedSequence(seqShared);
	
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
}
